@@ -994,6 +994,9 @@ gps_mask_t garmin_ser_parse(struct gps_device_t *session)
     unsigned char chksum = 0;
     gps_mask_t mask = 0;
 
+    if (session->packet.type == GARMIN_PACKET)
+	session->gpsdata.dev.driver_mode = MODE_BINARY;
+
     gpsd_report(LOG_RAW, "Garmin: garmin_ser_parse()\n");
     if (6 > len) {
 	/* WTF? */
@@ -1149,6 +1152,7 @@ static void garmin_switcher(struct gps_device_t *session, int mode)
 	    gpsd_report(LOG_IO,
 			"Garmin: => GPS: turn off binary %02x %02x %02x... \n",
 			switcher[0], switcher[1], switcher[2]);
+	    session->gpsdata.dev.driver_mode = MODE_NMEA;
 	} else {
 	    gpsd_report(LOG_ERROR, "Garmin: => GPS: FAILED\n");
 	}
@@ -1162,6 +1166,7 @@ static void garmin_switcher(struct gps_device_t *session, int mode)
     } else {
 	(void)nmea_send(session, "$PGRMC1,1,2,1,,,,2,W,N");
 	(void)nmea_send(session, "$PGRMI,,,,,,,R");
+	session->gpsdata.dev.driver_mode = MODE_BINARY;
 	settle();		// wait 333mS, essential!
     }
 }
