@@ -105,7 +105,7 @@ static void merge_hhmmss(char *hhmmss, struct gps_device_t *session)
     session->driver.nmea.date.tm_min = DD(hhmmss + 2);
     session->driver.nmea.date.tm_sec = DD(hhmmss + 4);
     session->driver.nmea.subseconds =
-	atof(hhmmss + 4) - session->driver.nmea.date.tm_sec;
+	safe_atof(hhmmss + 4) - session->driver.nmea.date.tm_sec;
 }
 
 static void register_fractional_time(const char *tag, const char *fld,
@@ -114,7 +114,7 @@ static void register_fractional_time(const char *tag, const char *fld,
     if (fld[0] != '\0') {
 	session->driver.nmea.last_frac_time =
 	    session->driver.nmea.this_frac_time;
-	session->driver.nmea.this_frac_time = atof(fld);
+	session->driver.nmea.this_frac_time = safe_atof(fld);
 	session->driver.nmea.latch_frac_time = true;
 	gpsd_report(LOG_DATA, "%s: registers fractional time %.2f\n",
 		    tag, session->driver.nmea.this_frac_time);
@@ -187,8 +187,8 @@ static gps_mask_t processGPRMC(int count, char *field[],
 	}
 	do_lat_lon(&field[3], &session->newdata);
 	mask |= LATLON_SET;
-	session->newdata.speed = atof(field[7]) * KNOTS_TO_MPS;
-	session->newdata.track = atof(field[8]);
+	session->newdata.speed = safe_atof(field[7]) * KNOTS_TO_MPS;
+	session->newdata.track = safe_atof(field[8]);
 	mask |= (TRACK_SET | SPEED_SET);
 	/*
 	 * This copes with GPSes like the Magellan EC-10X that *only* emit
@@ -367,13 +367,12 @@ static gps_mask_t processGPGGA(int c UNUSED, char *field[],
 	 * If we see this, force mode to 2D at most.
 	 */
 	if (altitude[0] == '\0') {
-	    if (session->newdata.mode == MODE_3D) {
-		session->newdata.mode =
-		    session->gpsdata.status ? MODE_2D : MODE_NO_FIX;
-		mask |= MODE_SET;
+	    if (session->newdata.mode > MODE_2D) {
+		session->newdata.mode = MODE_2D;
+                mask |= MODE_SET;
 	    }
 	} else {
-	    session->newdata.altitude = atof(altitude);
+	    session->newdata.altitude = safe_atof(altitude);
 	    mask |= ALTITUDE_SET;
 	    /*
 	     * This is a bit dodgy.  Technically we shouldn't set the mode
@@ -388,7 +387,7 @@ static gps_mask_t processGPGGA(int c UNUSED, char *field[],
 	    }
 	}
 	if (strlen(field[11]) > 0) {
-	    session->gpsdata.separation = atof(field[11]);
+	    session->gpsdata.separation = safe_atof(field[11]);
 	} else {
 	    session->gpsdata.separation =
 		wgs84_separation(session->newdata.latitude,
@@ -426,7 +425,7 @@ static gps_mask_t processGPGST(int count, char *field[], struct gps_device_t *se
       return 0;
     }
 
-#define PARSE_FIELD(n) (*field[n]!='\0' ? atof(field[n]) : NAN)
+#define PARSE_FIELD(n) (*field[n]!='\0' ? safe_atof(field[n]) : NAN)
     session->gpsdata.gst.utctime             = PARSE_FIELD(1);
     session->gpsdata.gst.rms_deviation       = PARSE_FIELD(2);
     session->gpsdata.gst.smajor_deviation    = PARSE_FIELD(3);
@@ -497,9 +496,9 @@ static gps_mask_t processGPGSA(int count, char *field[],
 	else
 	    mask = MODE_SET;
 	gpsd_report(LOG_PROG, "GPGSA sets mode %d\n", session->newdata.mode);
-	session->gpsdata.dop.pdop = atof(field[15]);
-	session->gpsdata.dop.hdop = atof(field[16]);
-	session->gpsdata.dop.vdop = atof(field[17]);
+	session->gpsdata.dop.pdop = safe_atof(field[15]);
+	session->gpsdata.dop.hdop = safe_atof(field[16]);
+	session->gpsdata.dop.vdop = safe_atof(field[17]);
 	session->gpsdata.satellites_used = 0;
 	memset(session->gpsdata.used, 0, sizeof(session->gpsdata.used));
 	/* the magic 6 here counts the tag, two mode fields, and the DOP fields */
@@ -647,11 +646,11 @@ static gps_mask_t processPGRME(int c UNUSED, char *field[],
 	mask = 0;
     } else {
 	session->newdata.epx = session->newdata.epy =
-	    atof(field[1]) * (1 / sqrt(2)) * (GPSD_CONFIDENCE / CEP50_SIGMA);
+	    safe_atof(field[1]) * (1 / sqrt(2)) * (GPSD_CONFIDENCE / CEP50_SIGMA);
 	session->newdata.epv =
-	    atof(field[3]) * (GPSD_CONFIDENCE / CEP50_SIGMA);
+	    safe_atof(field[3]) * (GPSD_CONFIDENCE / CEP50_SIGMA);
 	session->gpsdata.epe =
-	    atof(field[5]) * (GPSD_CONFIDENCE / CEP50_SIGMA);
+	    safe_atof(field[5]) * (GPSD_CONFIDENCE / CEP50_SIGMA);
 	mask = HERR_SET | VERR_SET | PERR_IS;
     }
 
@@ -686,9 +685,9 @@ static gps_mask_t processGPGBS(int c UNUSED, char *field[],
     if (session->driver.nmea.date.tm_hour == DD(field[1])
 	&& session->driver.nmea.date.tm_min == DD(field[1] + 2)
 	&& session->driver.nmea.date.tm_sec == DD(field[1] + 4)) {
-	session->newdata.epy = atof(field[2]);
-	session->newdata.epx = atof(field[3]);
-	session->newdata.epv = atof(field[4]);
+	session->newdata.epy = safe_atof(field[2]);
+	session->newdata.epx = safe_atof(field[3]);
+	session->newdata.epv = safe_atof(field[4]);
 	gpsd_report(LOG_DATA, "GBS: epx=%.2f epy=%.2f epv=%.2f\n",
 		    session->newdata.epx,
 		    session->newdata.epy,
@@ -723,7 +722,7 @@ static gps_mask_t processGPZDA(int c UNUSED, char *field[],
 
     if (field[1][0] == '\0' || field[2][0] == '\0' || field[3][0] == '\0'
 	|| field[4][0] == '\0') {
-	gpsd_report(LOG_WARN, "malformed ZDA\n");
+	gpsd_report(LOG_WARN, "ZDA fields are empty\n");
     } else {
     	int year, mon, mday, century;
 
@@ -786,7 +785,7 @@ static gps_mask_t processHDT(int c UNUSED, char *field[],
     gps_mask_t mask;
     mask = ONLINE_SET;
 
-    session->gpsdata.attitude.heading = atof(field[1]);
+    session->gpsdata.attitude.heading = safe_atof(field[1]);
     session->gpsdata.attitude.mag_st = '\0';
     session->gpsdata.attitude.pitch = NAN;
     session->gpsdata.attitude.pitch_st = '\0';
@@ -813,6 +812,53 @@ static gps_mask_t processHDT(int c UNUSED, char *field[],
     return mask;
 }
 
+static gps_mask_t processDBT(int c UNUSED, char *field[],
+				struct gps_device_t *session)
+{
+    /*
+     * $SDDBT,7.7,f,2.3,M,1.3,F*05
+     * 1) Depth below sounder in feet
+     * 2) Fixed value 'f' indicating feet
+     * 3) Depth below sounder in meters
+     * 4) Fixed value 'M' indicating meters
+     * 5) Depth below sounder in fathoms
+     * 6) Fixed value 'F' indicating fathoms
+     * 7) Checksum.
+     *
+     * In real-world sensors, sometimes not all three conversions are reported.
+     */
+    gps_mask_t mask;
+    mask = ONLINE_SET;
+
+    if (field[3][0] != '\0') {
+	session->newdata.altitude = -safe_atof(field[3]);
+	mask |= (ALTITUDE_SET);
+    } else if (field[1][0] != '\0') {
+	session->newdata.altitude = -safe_atof(field[1]) / METERS_TO_FEET;
+	mask |= (ALTITUDE_SET);
+    } else if (field[5][0] != '\0') {
+	session->newdata.altitude = -safe_atof(field[5]) / METERS_TO_FATHOMS;
+	mask |= (ALTITUDE_SET);
+    }
+
+    if ((mask & ALTITUDE_SET) != 0) {
+	if (session->newdata.mode < MODE_3D) {
+	    session->newdata.mode = MODE_3D;
+	    mask |= MODE_SET;
+	}
+    }
+
+    /*
+     * Hack: We report depth below keep as negative altitude because there's
+     * no better place to put it.  Should work in practice as nobody is 
+     * likely to be operating a depth sounder at varying altitudes.
+     */
+    gpsd_report(LOG_RAW, "mode %d, depth %lf.\n",
+		session->newdata.mode,
+		session->newdata.altitude);
+    return mask;
+}
+
 #ifdef TNT_ENABLE
 static gps_mask_t processTNTHTM(int c UNUSED, char *field[],
 				struct gps_device_t *session)
@@ -850,17 +896,17 @@ static gps_mask_t processTNTHTM(int c UNUSED, char *field[],
     gps_mask_t mask;
     mask = ONLINE_SET;
 
-    session->gpsdata.attitude.heading = atof(field[1]);
+    session->gpsdata.attitude.heading = safe_atof(field[1]);
     session->gpsdata.attitude.mag_st = *field[2];
-    session->gpsdata.attitude.pitch = atof(field[3]);
+    session->gpsdata.attitude.pitch = safe_atof(field[3]);
     session->gpsdata.attitude.pitch_st = *field[4];
-    session->gpsdata.attitude.roll = atof(field[5]);
+    session->gpsdata.attitude.roll = safe_atof(field[5]);
     session->gpsdata.attitude.roll_st = *field[6];
     session->gpsdata.attitude.yaw = NAN;
     session->gpsdata.attitude.yaw_st = '\0';
-    session->gpsdata.attitude.dip = atof(field[7]);
+    session->gpsdata.attitude.dip = safe_atof(field[7]);
     session->gpsdata.attitude.mag_len = NAN;
-    session->gpsdata.attitude.mag_x = atof(field[8]);
+    session->gpsdata.attitude.mag_x = safe_atof(field[8]);
     session->gpsdata.attitude.mag_y = NAN;
     session->gpsdata.attitude.mag_z = NAN;
     session->gpsdata.attitude.acc_len = NAN;
@@ -906,28 +952,28 @@ static gps_mask_t processOHPR(int c UNUSED, char *field[],
     gps_mask_t mask;
     mask = ONLINE_SET;
 
-    session->gpsdata.attitude.heading = atof(field[1]);
+    session->gpsdata.attitude.heading = safe_atof(field[1]);
     session->gpsdata.attitude.mag_st = '\0';
-    session->gpsdata.attitude.pitch = atof(field[2]);
+    session->gpsdata.attitude.pitch = safe_atof(field[2]);
     session->gpsdata.attitude.pitch_st = '\0';
-    session->gpsdata.attitude.roll = atof(field[3]);
+    session->gpsdata.attitude.roll = safe_atof(field[3]);
     session->gpsdata.attitude.roll_st = '\0';
     session->gpsdata.attitude.yaw = NAN;
     session->gpsdata.attitude.yaw_st = '\0';
     session->gpsdata.attitude.dip = NAN;
-    session->gpsdata.attitude.temp = atof(field[4]);
-    session->gpsdata.attitude.depth = atof(field[5]) / METERS_TO_FEET;
-    session->gpsdata.attitude.mag_len = atof(field[6]);
-    session->gpsdata.attitude.mag_x = atof(field[7]);
-    session->gpsdata.attitude.mag_y = atof(field[8]);
-    session->gpsdata.attitude.mag_z = atof(field[9]);
-    session->gpsdata.attitude.acc_len = atof(field[10]);
-    session->gpsdata.attitude.acc_x = atof(field[11]);
-    session->gpsdata.attitude.acc_y = atof(field[12]);
-    session->gpsdata.attitude.acc_z = atof(field[13]);
-    session->gpsdata.attitude.gyro_x = atof(field[15]);
-    session->gpsdata.attitude.gyro_y = atof(field[16]);
-    mask |= (ALTITUDE_SET);
+    session->gpsdata.attitude.temp = safe_atof(field[4]);
+    session->gpsdata.attitude.depth = safe_atof(field[5]) / METERS_TO_FEET;
+    session->gpsdata.attitude.mag_len = safe_atof(field[6]);
+    session->gpsdata.attitude.mag_x = safe_atof(field[7]);
+    session->gpsdata.attitude.mag_y = safe_atof(field[8]);
+    session->gpsdata.attitude.mag_z = safe_atof(field[9]);
+    session->gpsdata.attitude.acc_len = safe_atof(field[10]);
+    session->gpsdata.attitude.acc_x = safe_atof(field[11]);
+    session->gpsdata.attitude.acc_y = safe_atof(field[12]);
+    session->gpsdata.attitude.acc_z = safe_atof(field[13]);
+    session->gpsdata.attitude.gyro_x = safe_atof(field[15]);
+    session->gpsdata.attitude.gyro_y = safe_atof(field[16]);
+    mask |= (ATTITUDE_SET);
 
     gpsd_report(LOG_RAW, "Heading %lf.\n", session->gpsdata.attitude.heading);
     return mask;
@@ -965,14 +1011,14 @@ static gps_mask_t processPASHR(int c UNUSED, char *field[],
 	    merge_hhmmss(field[4], session);
 	    register_fractional_time(field[0], field[4], session);
 	    do_lat_lon(&field[5], &session->newdata);
-	    session->newdata.altitude = atof(field[9]);
-	    session->newdata.track = atof(field[11]);
-	    session->newdata.speed = atof(field[12]) / MPS_TO_KPH;
-	    session->newdata.climb = atof(field[13]);
-	    session->gpsdata.dop.pdop = atof(field[14]);
-	    session->gpsdata.dop.hdop = atof(field[15]);
-	    session->gpsdata.dop.vdop = atof(field[16]);
-	    session->gpsdata.dop.tdop = atof(field[17]);
+	    session->newdata.altitude = safe_atof(field[9]);
+	    session->newdata.track = safe_atof(field[11]);
+	    session->newdata.speed = safe_atof(field[12]) / MPS_TO_KPH;
+	    session->newdata.climb = safe_atof(field[13]);
+	    session->gpsdata.dop.pdop = safe_atof(field[14]);
+	    session->gpsdata.dop.hdop = safe_atof(field[15]);
+	    session->gpsdata.dop.vdop = safe_atof(field[16]);
+	    session->gpsdata.dop.tdop = safe_atof(field[17]);
 	    mask |= (TIME_SET | LATLON_SET | ALTITUDE_SET);
 	    mask |= (SPEED_SET | TRACK_SET | CLIMB_SET);
 	    mask |= DOP_SET;
@@ -994,7 +1040,7 @@ static gps_mask_t processPASHR(int c UNUSED, char *field[],
 	    session->gpsdata.PRN[i] = p = atoi(field[3 + i * 5 + 0]);
 	    session->gpsdata.azimuth[i] = atoi(field[3 + i * 5 + 1]);
 	    session->gpsdata.elevation[i] = atoi(field[3 + i * 5 + 2]);
-	    session->gpsdata.ss[i] = atof(field[3 + i * 5 + 3]);
+	    session->gpsdata.ss[i] = safe_atof(field[3 + i * 5 + 3]);
 	    if (field[3 + i * 5 + 4][0] == 'U')
 		session->gpsdata.used[u++] = p;
 	}
@@ -1024,13 +1070,14 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t * session)
     {
 	char *name;
 	int nf;			/* minimum number of fields required to parse */
+	bool cycle_continue;	/* cycle continuer? */
 	nmea_decoder decoder;
     } nmea_phrase[] = {
 	/*@ -nullassign @*/
-	{"PGRMC", 0, NULL},	/* ignore Garmin Sensor Config */
-	{"PGRME", 7, processPGRME}, 
-	{"PGRMI", 0, NULL},	/* ignore Garmin Sensor Init */
-	{"PGRMO", 0, NULL},	/* ignore Garmin Sentence Enable */
+	{"PGRMC", 0, false, NULL},	/* ignore Garmin Sensor Config */
+	{"PGRME", 7, false, processPGRME}, 
+	{"PGRMI", 0, false, NULL},	/* ignore Garmin Sensor Init */
+	{"PGRMO", 0, false, NULL},	/* ignore Garmin Sentence Enable */
 	    /*
 	     * Basic sentences must come after the PG* ones, otherwise
 	     * Garmins can get stuck in a loop that looks like this:
@@ -1040,29 +1087,32 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t * session)
 	     * 2. PGRMC is sent to reconfigure to Garmin binary mode.
 	     *    If successful, the GPS echoes the phrase.
 	     *
-	     * 3. nmea_parse() sees the echo as RMC because the talker ID is
-	     *    ignored, and fails to recognize the echo as PGRMC and ignore it.
+	     * 3. nmea_parse() sees the echo as RMC because the talker
+	     *    ID is ignored, and fails to recognize the echo as
+	     *    PGRMC and ignore it.
 	     *
-	     * 4. The mode is changed back to NMEA, resulting in an infinite loop.
+	     * 4. The mode is changed back to NMEA, resulting in an
+	     *    infinite loop.
 	     */
-	{"RMC", 8,  processGPRMC},
-	{"GGA", 13, processGPGGA},
-	{"GST", 8,  processGPGST},
-	{"GLL", 7,  processGPGLL},
-	{"GSA", 17, processGPGSA},
-	{"GSV", 0,  processGPGSV},
-	{"VTG", 0,  NULL},	/* ignore Velocity Track made Good */
-	{"ZDA", 4,  processGPZDA},
-	{"GBS", 7,  processGPGBS},
-        {"HDT", 1,  processHDT},
+	{"RMC", 8,  false, processGPRMC},
+	{"GGA", 13, false, processGPGGA},
+	{"GST", 8,  false, processGPGST},
+	{"GLL", 7,  false, processGPGLL},
+	{"GSA", 17, false, processGPGSA},
+	{"GSV", 0,  false, processGPGSV},
+	{"VTG", 0,  false, NULL},	/* ignore Velocity Track made Good */
+	{"ZDA", 4,  false, processGPZDA},
+	{"GBS", 7,  false, processGPGBS},
+        {"HDT", 1,  false, processHDT},
+	{"DBT", 7,  true,  processDBT},
 #ifdef TNT_ENABLE
-	{"PTNTHTM", 9, processTNTHTM},
+	{"PTNTHTM", 9, false, processTNTHTM},
 #endif /* TNT_ENABLE */
 #ifdef ASHTECH_ENABLE
-	{"PASHR", 3, processPASHR},	/* general handler for Ashtech */
+	{"PASHR", 3, false, processPASHR},	/* general handler for Ashtech */
 #endif /* ASHTECH_ENABLE */
 #ifdef OCEANSERVER_ENABLE
-	{"OHPR", 18, processOHPR},
+	{"OHPR", 18, false, processOHPR},
 #endif /* OCEANSERVER_ENABLE */
 	    /*@ +nullassign @*/
     };
@@ -1141,6 +1191,8 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t * session)
 		(void)strlcpy(session->gpsdata.tag, 
 			      nmea_phrase[i].name, 
 			      MAXTAGLEN);
+		if (nmea_phrase[i].cycle_continue)
+		    session->driver.nmea.cycle_continue = true; 
 		/*
 		 * Must force this to be nz, as we're going to rely on a zero
 		 * value to mean "no previous tag" later.
@@ -1182,7 +1234,7 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t * session)
     /*
      * The end-of-cycle detector.  This code depends on just one
      * assumption: if a sentence with a timestamp occurs just before
-     * start of cycle, then it is always good to trigger a reort on
+     * start of cycle, then it is always good to trigger a report on
      * that sentence in the future.  For devices with a fixed cycle
      * this should work perfectly, locking in detection after one
      * cycle.  Most split-cycle devices (Garmin 48, for example) will
@@ -1192,7 +1244,7 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t * session)
      */
     if (session->driver.nmea.latch_frac_time) {
 	gpsd_report(LOG_PROG,
-		    "%s reporting cycle started on %.2f.\n",
+		    "%s sentence timestamped %.2f.\n",
 		    session->driver.nmea.field[0],
 		    session->driver.nmea.this_frac_time);
 	if (!GPS_TIME_EQUAL
@@ -1206,25 +1258,42 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t * session)
 	    /*
 	     * Have we seen a previously timestamped NMEA tag?
 	     * If so, designate as end-of-cycle marker.
+	     * But not if there are continuation sentences;
+	     * those get sorted after the last timestamped sentence
 	     */
 	    if (lasttag > 0
-		&& (session->driver.nmea.cycle_enders & (1 << lasttag)) ==
-		0) {
+		&& (session->driver.nmea.cycle_enders & (1 << lasttag)) == 0
+		&& !session->driver.nmea.cycle_continue) {
 		session->driver.nmea.cycle_enders |= (1 << lasttag);
 		gpsd_report(LOG_PROG,
 			    "tagged %s as a cycle ender.\n",
 			    nmea_phrase[lasttag - 1].name);
 	    }
 	}
-	/* here's where we check for end-of-cycle */
-	if (session->driver.nmea.cycle_enders & (1 << thistag)) {
+    } else {
+	/* extend the cycle to an un-timestamped sentence? */ 
+	if ((session->driver.nmea.lasttag & session->driver.nmea.cycle_enders) != 0)
 	    gpsd_report(LOG_PROG,
-			"%s ends a reporting cycle.\n",
+			"%s is just after a cycle ender.\n",
 			session->driver.nmea.field[0]);
-	    retval |= REPORT_IS;
+	if (session->driver.nmea.cycle_continue) {
+	    gpsd_report(LOG_PROG,
+			"%s extends the reporting cycle.\n",
+			session->driver.nmea.field[0]);
+	    session->driver.nmea.cycle_enders &=~ (1 << session->driver.nmea.lasttag);
+	    session->driver.nmea.cycle_enders |= (1 << thistag);
 	}
-	session->driver.nmea.lasttag = thistag;
     }
+    /* here's where we check for end-of-cycle */
+    if ((session->driver.nmea.latch_frac_time || session->driver.nmea.cycle_continue)
+	&& (session->driver.nmea.cycle_enders & (1 << thistag))!=0) {
+	gpsd_report(LOG_PROG,
+		    "%s ends a reporting cycle.\n",
+		    session->driver.nmea.field[0]);
+	retval |= REPORT_IS;
+    }
+    if (session->driver.nmea.latch_frac_time)
+	session->driver.nmea.lasttag = thistag;
 
     /* we might have a reliable end-of-cycle */
     if (session->driver.nmea.cycle_enders != 0)
