@@ -693,11 +693,11 @@ struct subframe_t {
 	    uint8_t IODE;
 	    /* Rate of Inclination Angle, 14 bits signed, scale2**-43,
 	     * semi-circles/sec */
-	    uint16_t IDOT;
+	    int16_t IDOT;
 	    double d_IDOT;
 	    /* Cic, Amplitude of the Cosine Harmonic Correction Term to the 
 	     * Angle of Inclination, 16 bits signed, scale 2**-29, radians*/
-	    uint16_t Cic;
+	    int16_t Cic;
 	    double d_Cic;
 	    /* Cis, Amplitude of the Sine Harmonic Correction Term to the
 	     * Angle of Inclination, 16 bits, unsigned, scale 2**-29, radians */
@@ -1753,6 +1753,7 @@ extern double wgs84_separation(double, double);
 /* some multipliers for interpreting GPS output */
 #define METERS_TO_FEET	3.2808399	/* Meters to U.S./British feet */
 #define METERS_TO_MILES	0.00062137119	/* Meters to miles */
+#define METERS_TO_FATHOMS	0.54680665	/* Meters to fathoms */
 #define KNOTS_TO_MPH	1.1507794	/* Knots to miles per hour */
 #define KNOTS_TO_KPH	1.852		/* Knots to kilometers per hour */
 #define KNOTS_TO_MPS	0.51444444	/* Knots to meters per second */
