@@ -167,7 +167,7 @@ static void gpsd_binary_quality_dump(struct gps_device_t *session,
     char *bufp2 = bufp;
     bool used_valid = (session->gpsdata.set & USED_IS) != 0;
 
-    if (session->device_type != NULL && (session->gpsdata.set & MODE_IS) != 0) {
+    if (session->device_type != NULL && (session->gpsdata.set & MODE_SET) != 0) {
 	(void)snprintf(bufp, len - strlen(bufp),
 		       "$GPGSA,%c,%d,", 'A', session->gpsdata.fix.mode);
 	j = 0;
@@ -281,17 +281,17 @@ void nmea_tpv_dump(struct gps_device_t *session,
 		   /*@out@*/ char bufp[], size_t len)
 {
     bufp[0] = '\0';
-    if ((session->gpsdata.set & TIME_IS) != 0)
+    if ((session->gpsdata.set & TIME_SET) != 0)
 	gpsd_binary_time_dump(session, bufp + strlen(bufp),
 			      len - strlen(bufp));
-    if ((session->gpsdata.set & LATLON_IS) != 0) {
+    if ((session->gpsdata.set & LATLON_SET) != 0) {
 	gpsd_position_fix_dump(session, bufp + strlen(bufp), 
 			       len - strlen(bufp));
 	gpsd_transit_fix_dump(session, bufp + strlen(bufp),
 			      len - strlen(bufp));
     }
     if ((session->gpsdata.set
-	 & (MODE_IS | DOP_IS | USED_IS | HERR_IS | VERR_IS)) != 0)
+	 & (MODE_SET | DOP_SET | USED_IS | HERR_SET | VERR_SET)) != 0)
 	gpsd_binary_quality_dump(session, bufp + strlen(bufp),
 				 len - strlen(bufp));
 }
@@ -301,7 +301,7 @@ void nmea_sky_dump(struct gps_device_t *session,
 		   /*@out@*/ char bufp[], size_t len)
 {
     bufp[0] = '\0';
-    if ((session->gpsdata.set & SATELLITE_IS) != 0)
+    if ((session->gpsdata.set & SATELLITE_SET) != 0)
 	gpsd_binary_satellite_dump(session, bufp + strlen(bufp),
 				   len - strlen(bufp));
 }
@@ -310,7 +310,7 @@ void nmea_subframe_dump(struct gps_device_t *session,
 		   /*@out@*/ char bufp[], size_t len)
 {
     bufp[0] = '\0';
-    if ((session->gpsdata.set & SUBFRAME_IS) != 0)
+    if ((session->gpsdata.set & SUBFRAME_SET) != 0)
 	gpsd_binary_almanac_dump(session, bufp + strlen(bufp),
 				   len - strlen(bufp));
 }
